from cli._version import __version__
import click
from cli.plugins import _plugins
from cli.dictionary import dictionary
from cli.utils import Config, TableDisplay,ColorText
import os,json
from mymodule import mkdirs
from cli.plugins import plugin_folder
import requests
from datetime import datetime
import socket

td = TableDisplay()

def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(__version__)
    ctx.exit()

def export_config(ctx,param,path):
    if path is None or ctx.resilient_parsing:
        return
    result = extract_settings('conf')
    if path.endswith('-'):
        newtd=TableDisplay(color=ColorText({"[]":"fC"}))
        click.echo(newtd(title= ">>>[ Config Parameters ]<<<" ,text=json.dumps(result,indent=2).strip('{}') ))
    else:
        mkdirs(path)
        filepath = os.path.join(path,'OK_config.json')
        with open(filepath,'wt') as f:
            json.dump(result,f,indent=2)
        click.echo(td(text=f' Config data Saved! \n File saved to:\n [{filepath}]'))
    ctx.exit()

def extract_settings(settings):
    """
    extract all config files to a dictionary
    """
    data = {}
    if settings == 'conf':
        for name in Config.list_config():
            if name != 'sync_settings':
                data[name] = Config(name).readData()
    elif settings =='plugin':
        for filename in os.listdir(plugin_folder):
            if filename.endswith('.py'):
                with open(os.path.join(plugin_folder, filename), 'rt') as f:
                    data[filename] = f.read()
    return data

def write_settings(data,settings):
    """
    write all the config files to conf folder.
    """
    if settings == 'conf':
        for name,settings in data.items():
            if name!='sync_settings':
                config=Config(name)
                config.saveData(settings)
                click.echo(td(text=f"<g>Success!</g> <y>{name}</y> configuration imported."))
    elif settings == 'plugin':
        for name, txt in data.items():
            with open(os.path.join(plugin_folder, name), 'wt') as f:
                f.write(txt)
            click.echo(
                td(text=f"<g>Success!</g> Plugin <y>{name[:-3]}</y> imported."))




def write_gist(data,auth,gist,settings):
    """
    write a data dict to gist.
    settings = 'plugin' or 'config'
    """
    if not gist:
        url = "https://api.github.com/gists"
    else:
        url = f"https://api.github.com/gists/{gist}"
    headers = {
        'Authorization': f'token {auth}',
        'Content-Type': 'applicaiton/json'
    }
    payload = {
        "description": f"OK tools settings configuration and plugins",
        "public": True,
        "files": {
            f"{settings}.json": {
                "content": json.dumps(data,indent=2),
            },
            "OK_config_lastupdate" : {
                "content": f'UPDATE ON: {datetime.now().strftime("%c")}, FROM: {socket.gethostname()}'
            }
        }
    }
    try:
        if gist:
            res = requests.patch(url,headers=headers,data = json.dumps(payload))
            if res.status_code == 200:
                click.echo(
                    td(text=f"<g>Successfully synced</g> <y>{settings}</y> <g>settings!</g>\nGist @ [https://gist.github.com/{gist}]"))

        else:
            res = requests.post(url,headers=headers,data=json.dumps(payload))
            if res.status_code == 201:
                gist = res.json()['id']
                update_Sync_settings(gist=gist)
                click.echo(
                    td(text=f"<g>Successfully CREATED NEW</g> <y>{settings}</y> <g>settings!</g>\nGist @ [https://gist.github.com/{gist}]"))
        if res.status_code not in [200,201]:
            jsontd = TableDisplay(color=ColorText({"[]": "fR"}))
            click.echo(jsontd(title=">>> [Sync failed with following response] <<<",
                          text=json.dumps(res.json(), indent=2).strip("{}")))
    except Exception as e:
        click.echo(td(title=">>> <r> ! Sync Failed </r><<<", text=f"{e}"))

def update_Sync_settings(gist= None, auth=None ):
    config = Config('sync_settings')
    data = config.readData()
    if gist is not None:
        data.update(gist=gist.strip())
        click.echo(td(text=f"Saved Gist ID: [<{gist}>]."))
    if auth is not None:
        data.update(auth=auth.strip())
        click.echo(td(text=f"Saved auth: [<{auth}>]."))
    config.saveData(data)


def extract_gist(gist,settings):
    url = f"https://api.github.com/gists/{gist}"
    try:
        res = requests.request('GET',url,).json()
        if res.get('message',None):
            raise ValueError (f"Failed to load gist: [{url}]\nReason: <a> {res.get('message')} </a>")
        data = json.loads(res['files'][f'{settings}.json']['content'])
        return data
    except Exception as e:
        click.echo(td(title= ">>> <r> ! Import Failed </r><<<" ,text=f"{e}"))
        return None

def import_config(ctx,param,value):
    """
    from a file or url
    "url": "https://api.github.com/gists/02641f8b5a9e9c0de1a9b2a6825d3f60",
    """
    if value is None or ctx.resilient_parsing:
        return
    data = json.load(value)
    write_settings(data,settings='conf')
    ctx.exit()

def execute_sync(direction,option):
    """
    run the sync according to user select.
    direction: up or down
    option: conf or plugin
    sync_settings: dictionary for sync_settings.json, contain gist id and auth.
    """
    config = Config('sync_settings')
    sync_settings = config.readData()
    gist = sync_settings.get('gist', None)
    auth = sync_settings.get('auth', None)
    if direction == 'up':
        if not auth:
            click.echo(
                td(text="<a> Set up Auth Token to Upload settings. </a>\nRefer to: [https://github.com/settings/tokens]"))
            return
        data = extract_settings(option)
        write_gist(data,auth,gist,option)
        return
    elif direction == 'down':
        if not gist:
            click.echo(
                td(text="<a> Set up gist ID to download settings. </a>\nRefer to: [https://gist.github.com/]"))
            return
        data = extract_gist(gist,option)
        write_settings(data,option)
        return


@click.group(cls=click.CommandCollection)
@click.option('--version','-v', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True,help='Show version.')
@click.option('--export-config',type=click.Path(allow_dash=True,resolve_path=True), callback=export_config,
              expose_value=False, is_eager=True,help='Export config to a file.')
@click.option('--import-config',type=click.File('r'),callback=import_config,
              expose_value=False, is_eager=True,help='Import config from a file.' )
@click.pass_context
def menu(ctx):
    "Main Entrance for plugins. And dictionary."
    pass

menu.sources = [_plugins, dictionary]


@_plugins.command()
@click.option('--set-auth','-sa','auth',default=None,help="Set Github Auth Token for sync.")
@click.option('--set-gist','-sg','gist',default=None,help="Set Github Gist ID for sync.")
@click.option('--upload','-u','direction',flag_value='up',help="Upload settings.")
@click.option('--download','-d','direction',flag_value='down',help="Download settings.")
@click.option('--plugin','-p','option',flag_value='plugin',help="Sync plugins.")
@click.option('--conf','-c','option',flag_value='conf',help="Sync configuration.")
@click.pass_context
def sync(ctx,auth,direction,option,gist):
    """
    Sync plugins or config with a gist.
    """
    # for syncing: need gist, auth. if only gist, can download,
    showhelp = True
    if (auth is not None) or (gist is not None):
        showhelp = False
        update_Sync_settings(auth=auth,gist=gist)

    if direction:
        if not option:
            pp = td(
                text=f"No sync option specified. \nDo you want to sync <g>both plugin and conf</g>?") + '\n'
            click.confirm(pp,default=True,abort=True)
            execute_sync(direction,'plugin')
            execute_sync(direction,'conf')
        else:
            execute_sync(direction,option)
        ctx.exit()
    if option:
        pp = td(text=f"You are going to sync >> [{option}] <<\n<r>No syncing direction specified.</r>\nYou must specify upload / download before continue.") + '\n'
        direction = click.prompt( pp,type=click.Choice(['up','down'],case_sensitive=False))
        execute_sync(direction, option)
        ctx.exit()

    if showhelp:
        click.echo(ctx.get_help())



import click
import subprocess as sub
from datetime import datetime as date
import socket
from cli.utils import TableDisplay

@click.group(invoke_without_command=True)
@click.pass_context
def cli(ctx):
    """
    git pull add commit push 4 step in one.
    """
    if ctx._depth == 2:
        click.echo('\nGit pull')
        sub.run('git pull',shell=True,)
        click.echo('\nCommit to Git')
        msg = click.prompt("Enter a comment",show_default=True,
            default=f"ON: {date.now().strftime('%c')}, FROM: {socket.gethostname()}")
        sub.run(f'git add . \n git commit -m "{msg}" \n git push',shell=True)
        result = sub.run('git config --get remote.origin.url',stdout=sub.PIPE,encoding='utf-8',shell=True).stdout
        td = TableDisplay()
        click.echo(td(title='>>> <g>! Success</g> <<<',text=f"Commited to [{result.strip()}]"))

README="# This Repo is created by [OK git][huik]\n[![python version](https://img.shields.\
io\/badge/python-3.5%20%7C%203.6%20%7C%203.7%20-blue)][pythonwebsite]\n[![Alt text]\
(https://img.shields.io/pypi/v/huik-module 'Hover to see this text.')][huik]\n### \
Emphasis\n*Italic* ; **Bold** ; ***Bold and Italic*** ; ~~Scratch~~\n### List items\n1. \
First ordered list item\n* Unordered list can use asterisks\n\n[huik]:https://pypi.org\
/project/huik-module\n[pythonwebsite]: https://www.python.org/downloads/release/python-375\n"

IGNORE={
"py": '*.py[cod]\n__pycache__\n*.so\n.env\n*.json\ntest*.*\ntest*\n*.bak\n*.dat\n*.dir\
\n*.egg\n*.egg-info\ndump.rdb\ndist\nbuild\neggs\nparts\nbin\nvar\nsdist\ndevelop-eggs\n\
.installed.cfg\nlib\nlib64\n.DS_Store\npip-log.txt\n.coverage\n.tox\nnosetests.xml\n*.vscode\
\n*.code-workspace\n*.mo\n.mr.developer.cfg\n.project\n.pydevproject\n.env\nvenv\nlogs\n',
"js":'/node_modules\n/.pnp\n.pnp.js\n/coverage\n/build\n.DS_Store\n.env.local\n\
.env.development.local\n.env.test.local\n.env.production.local\nnpm-debug.log*\
\nyarn-debug.log*\nyarn-error.log*\n'
}

@cli.command()
@click.option('--javascript','-js','project',flag_value='js',help='Setup javascript .gitignore')
@click.option('--python','-py','project',flag_value='py',help='Setup python .gitignore')
def init(project):
    "Init a new git package with .gitignore and README.md"
    if not project:
        project =  click.prompt("Type of project to init",
        show_default=True,type=click.Choice(["py","js"],case_sensitive=False),
        default="py")
    with open('README.md','wt') as f:
        f.write(README)
    with open('.gitignore','wt') as f:
        f.write(IGNORE[project])
    url = click.prompt("Enter github repo url")
    sub.run(f'git init\ngit add .\ngit commit -m "First Commit"\ngit remote add origin {url}\ngit push --set-upstream origin master',shell=True)
   
    
    


import click
import requests
from cli.utils import ColorText,TableDisplay,Config
import json

APIs = Config('api')
_data = APIs.readData()
COL_API_KEY=_data.get("COL_API_KEY",None)
MED_API_KEY=_data.get("MED_API_KEY",None)
URBAN_API_KEY=_data.get("URBAN_API_KEY",None)

def lookupMW(word,limit):
    url1=f"https://www.dictionaryapi.com/api/v3/references/collegiate/json/{word}?key={COL_API_KEY}"
    url2=f"https://www.dictionaryapi.com/api/v3/references/medical/json/{word}?key={MED_API_KEY}"
    try:
        if COL_API_KEY is None or (MED_API_KEY is None):
            raise ValueError ("Missing API key for M-W dictionary. Run def --config to setup.")
        r1 = requests.request("GET", url1, ).json()
        r2 = requests.request("GET", url2, ).json()
        def process(r):
            result = []
            for k,d in enumerate(r[0:limit]):
                if not isinstance(d,dict):
                    continue
                _def = [("<gDef {}.g> [{}]".format(k+1,d.get('fl','None')))]
                for i,l in enumerate(d.get('shortdef',[])):
                    _def.append(f"  [{i+1}]. {l}")
                result.append(_def)
            return result
        res1 = process(r1)
        res2 = process(r2)
        result = [["{From M-W Collegiate Dictionary}"]]+res1+[["{From M-W Medical Dictionary}"]]+res2

        return result
    except Exception as e:
        return e


def lookupUrban(word,limit):
    """
    look up word on urban dictionary, export a upper limit
    """
    url = "https://mashape-community-urban-dictionary.p.rapidapi.com/define"
    querystring = {"term":word}
    headers = {
        'x-rapidapi-host': "mashape-community-urban-dictionary.p.rapidapi.com",
        'x-rapidapi-key': URBAN_API_KEY
        }
    try:
        if URBAN_API_KEY is None:
            raise ValueError ("No API key for urban dictionary. Run def --config to setup.")
        response = requests.request("GET", url, headers=headers, params=querystring)
        r=response.json()
        result = []
        for k,d in enumerate(r['list'][0:limit]):
            _def = [("<gDef {}.g> {}".format(k+1,d.get('definition',"None")))]
            txt    =[f'<gExample:g>']
            exp = list((d.get('example',"None")).split('\n'))
            result.append(_def+txt+exp)
        return result
    except Exception as e:
        return e

def run_config():
    ck = click.prompt("Enter Key for M-W collegiate dictionary",default=COL_API_KEY)
    mk = click.prompt("Enter Key for M-W medical dictionary",default=MED_API_KEY)
    uk = click.prompt("Enter Key for Urban dictionary",default=URBAN_API_KEY)
    data = APIs.readData()
    newdata = dict(COL_API_KEY=ck,MED_API_KEY=mk,URBAN_API_KEY=uk)
    data.update(newdata)
    click.echo('You have entered these keys:')
    click.echo(json.dumps(newdata,indent=4),)
    if click.confirm('Do you want to save?',abort=True,default=True):
        APIs.saveData(data)
    click.echo('API keys are saved.')

@click.group()
def dictionary():
    pass

@dictionary.command('def')
@click.argument("word",nargs=-1)
@click.option('-u','--urban','-urban','dictionary',flag_value="urban",default=False,help="Use Urban Dictionary")
@click.option('-mw','--mw','-m','-mw','dictionary',flag_value="mw",default=True,help="Use M-W Dictionary")
@click.option("--dictionary",'-d',type=click.Choice(['mw',"urban"],case_sensitive=False),default='mw',help="Select M-W or Urban Dictionary")
@click.option("--limit",'-l',default=5,help="Limit explanation entries")
@click.option("--config",is_flag=True,help="Configure Dictionary API keys")
@click.pass_context
def _def(ctx,word,limit,dictionary,config):
    """
    Dictionary. Support Urban Dict / M-W Dict.
    """
    if config:
        run_config()
        ctx.exit()
    if not word:
        click.echo(ctx.get_help())
        word = click.prompt('Please enter a word', type=str)
    else:
        word = " ".join(word)
    bkcolor = ColorText(mapping={"[]":"fC",("<g","g>") : "fG",("<alert>","</alert>"):"bR","{}":"bB"})
    if dictionary == "urban":
        result = lookupUrban(word,limit)
    else:
        result = lookupMW(word,limit)
    if isinstance(result,list) and result:
        td=TableDisplay(bkcolor)
        click.echo("")
        title = " DEFINITION : <g"+f">>> {word.capitalize()} <<<g>"
        click.echo(td.format(title=title,text=result))
        click.echo("")
    else:
        click.echo("")
        click.echo(bkcolor(f'<alert>!Unable to find online definition. \n <{result}> </alert>'))
        click.echo("")



from inspect import signature
import timeit
import psutil
import multiprocessing
from itertools import product

class FT_Decorator():
    def __init__(self,freq=1,callback=None,show=True):
        self.freq = freq
        self.callback = callback
        self.avgtime = 0
        self.count = 0
        self.show  = show

    def __call__(self,func):
        """
        decorator for printing execution time of a function.
        """
        sig = signature(func)
        def wrapped(*args, **kwargs):
            t1 = timeit.default_timer()
            result = func(*args, **kwargs)
            t2 = timeit.default_timer()


            self.avgtime = (self.count * self.avgtime + t2 - t1 ) / (self.count + 1)
            self.count += 1

            if self.callback:
                self.callback(t2-t1)

            if self.show and (self.count % self.freq == 0):
                print('Run {} {} times: avg: {:.5f}s; latest: {:.5f}s'.format(
                    func.__name__, self.count, self.avgtime, t2-t1))
            return result
        wrapped.__signature__ = sig
        wrapped.__name__ = func.__name__
        return wrapped

def ft_decorator(freq=1):
    def _ft_decorator(func):
        """
        decorator for printing execution time of a function.
        """
        sig = signature(func)
        avgtime = 0
        runcount = 0
        def wrapped(*args, **kwargs):
            nonlocal avgtime, runcount
            t1 = timeit.default_timer()
            result = func(*args, **kwargs)
            t2 = timeit.default_timer()
            avgtime = (runcount * avgtime + t2 - t1 ) / (runcount + 1)
            runcount += 1
            if runcount % freq == 0:
                print('Run {} {} times: avg: {:.5f}s; para:{}{}'.format(
                    func.__name__, runcount, avgtime, args, kwargs))
            return result
        wrapped.__signature__ = sig
        wrapped.__name__ = func.__name__
        return wrapped
    return _ft_decorator

def ft(func,args,kwargs={}, number=100):
    """
    use timeit to time [number] execution of function.
    """
    def wrapper():
        return func(*args,**kwargs)
    t = timeit.timeit(wrapper, number=number)
    print('Run {} {} times: total {:.6f}s; average {:.6f}s.'.format(func.__name__, number, t,t/number))
    return t

def poolMap(task,workload,initializer=None,initargs=None,chunks=None,
            total=None,progress_callback=None,progress_gap=(0,100),**kwargs):
    """
    speed up task in a list by multiprocessing.
    task is the function to apply on workload.
    workload is a iterable containing the task inputs.
    initializerr and initargs can be used to setup the function.
    """
    workerheads=psutil.cpu_count(logical=False)
    worker=multiprocessing.Pool(workerheads,initializer,initargs)
    total = total or len(workload)
    chunksize= int(total//chunks)+1 if chunks else int(total/workerheads/10+1)
    result = []
    count=0
    progress = progress_gap[0]
    for _ in worker.imap(task, workload, chunksize):
      count+=1
      result.append(_)
      if progress_callback :
          current_pro = count/total*(progress_gap[1]-progress_gap[0])+progress_gap[0]
          if current_pro > progress + 1:
              progress = current_pro
              progress_callback(current_pro)
    worker.close()
    worker.join()
    worker.terminate()
    return result


def cartesian(*arrays):
    """
    Give a list of arrays, return iterator of product of elements from arrays.
    similar to python product, but also yield indexes.
    """
    lenths = [len(i) for i in arrays]
    l = product(*lenths)
    p = product(*arrays)
    for i,j in zip(l,p):
        yield i,j



class MyPrint:
    """
    use mprint the same way as print().
    Set printToScreen = False to disable print. 
    Set mprint.callback to provide a callabck(msg) function when printToScreen is disabled.
    by default, call back is void().
    """
    printToScreen = True
  

    def callback(self,msg):
        return 
        
    def __call__(self,msg):
        if self.printToScreen:
            print(msg)
        else:
            self.callback(msg)
   
mprint = MyPrint()



REVSECOMPDICT = {'A': 'T',
     'T': 'A',
     'C': 'G',
     'G': 'C',
     'N': 'N',
     'a': 'T',
     't': 'A',
     'c': 'G',
     'g': 'C',
     'n': 'N'}

def revcomp(s):
    """
    return reverse complement of a sequence.
    """
    # define the dic out side is much faster.    
    comp = [REVSECOMPDICT[i] for i in s[::-1]]
    # comp = map(lambda x: REVSECOMPDICT[x], s[::-1]) # slightly slower
    return ''.join(comp)


def lev_distance(s1, s2, threshold=1000):
    """
    calculate lev_distance of s1 and s2.
    use threshold to calculate diagonally.stop at threshold. fastest.
    """
    if len(s1) > len(s2):
        s1, s2 = s2, s1
    l1 = len(s1)
    vertical = [0]
    horizontal = [0]
    for i in range(l1):
        char1 = s1[i]
        char2 = s2[i]
        newvertical = [i+1]
        newhorizontal = [i+1]
        for k in range(i):
            if char1 == s2[k]:
                newvertical.append(vertical[k])
            else:
                newvertical.append(
                    1+min(newvertical[-1], vertical[k], vertical[k+1]))
            if char2 == s1[k]:
                newhorizontal.append(horizontal[k])
            else:
                newhorizontal.append(
                    1+min(newhorizontal[-1], horizontal[k], horizontal[k+1]))
        last = vertical[-1] if char1 == char2 else (
            1+min(newvertical[-1], newhorizontal[-1], vertical[-1]))
        newhorizontal.append(last)
        newvertical.append(last)
        currentmin = min(min(newhorizontal), min(newvertical))
        if currentmin > threshold:
            return currentmin
        vertical, horizontal = newvertical, newhorizontal
    horizontal.append(last)
    for index2, char2 in enumerate(s2[l1:]):
        newhorizontal = [index2+l1+1]
        for index1, char1 in enumerate(s1):
            if char1 == char2:
                newhorizontal.append(horizontal[index1])
            else:
                newhorizontal.append(1 + min((horizontal[index1],
                                              horizontal[index1 + 1],
                                              newhorizontal[-1])))
        currentmin = min(newhorizontal)
        if currentmin > threshold:
            return currentmin
        horizontal = newhorizontal
    return horizontal[-1]



import click
import subprocess
from cli.utils import  TableDisplay, Config
from cli._version import __version__
from cli.ok import print_version

TL_CONFIG = Config('tools')

td = TableDisplay()

def displayMenu(data):
    dis = []
    for k in sorted(data.keys()):
        dis.append(f" >><g> [ {k} ] </g><< {data[k]['name']}  ")
    click.echo(td(title= "[>>Tools Menu<<]", text=[dis]))

def run_tool(key,data):
    """
    key is the key to command dictionary sotred in data dict.
    data dict: key:{name: ... , command: ...}
    """
    if not key:
        click.echo(td(text="<y>No option selected.</y>"))
        return
    if key in data:
        name = data[key]['name']
        command = data[key]['command']
        click.echo(td(text=f"Run [{name}]\n$ <i>{command}</i>"))
        subprocess.run(command,shell=True)
    else:
        click.echo(td(text=f" <r>'{key}'</r> <y>not in menu.</y>"))
        return

@click.command()
@click.option('--version','-v', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True,help='Show version.')
@click.option('-a','--add','ops',flag_value='add', help="Add new command.")
@click.option('-d','--delete','ops',flag_value='delete',help="Delete a command.")
@click.option('-e','--edit','ops',flag_value='edit',help="Edit a command.")
@click.option('-o','--open','ops',flag_value='open',help="Open command configure in editor.")
@click.argument('arg',nargs=-1,default=None)
@click.pass_context
def toollist(ctx,arg,ops):
    """
    Open Tools menu  by $ tl\n
    Directly invoke tool list command by $ tl [command key].\n
    Configure tools list by $ tl -a/-e/-d/-o.
    """
    if ops:
        config(ops)
        ctx.exit()

    data = TL_CONFIG.readData()
    if not arg:
        displayMenu(data)
        key = click.prompt('Enter Key',default="",show_default=False)
        return run_tool(key,data)

    for key in arg:
        run_tool(key,data)


def config(ops):
    """
    To configure the tool list menu.
    """
    if ops == "open":
        subprocess.run(f"open {TL_CONFIG.path}",shell=True)
        return

    data = TL_CONFIG.readData()
    if ops == "add":
        click.echo("Add a New Command")
        name = click.prompt('Enter Command Name', type=str)
        click.echo(f"You Entered: {name}\n")
        command = click.prompt('Enter Command Script',type=str)
        click.echo(f"You Entered: {command}\n")
        key = click.prompt('Enter Associated Key',type=str)
        click.echo(f"You Entered: {key}\n")
        if click.confirm('Do you want to save?',abort=True,default=True):
            temp = key
            while temp in data:
                temp+="d"
            if temp != key:
                data[temp]=data.pop(key)
            data[key]={'name':name,'command':command}

    elif ops == "delete":
        displayMenu(data)
        keys = click.prompt('Enter the command you wan to delete',type=str)
        for key in keys.split():
            if key in data:
                name = data.pop(key)['name']
            else:
                click.echo(td(title="",text=[[f"<a> '{key}' not in menu.</a>"]]))
        return
    elif ops == 'edit':
        displayMenu(data)
        key = click.prompt('Enter the command you wan to edit',type=str)
        if key not in data:
            click.echo(td(title="",text=[[f"<a> '{key}' not in menu.</a>"]]))
            return
        edit = data[key]
        name = click.prompt('Enter New Command Name', default=edit['name'])
        click.echo(f"You Entered: {name}\n")
        command = click.prompt('Enter New Command Script',default=edit['command'])
        click.echo(f"You Entered: {command}\n")
        newkey = click.prompt('Enter New Associated Key',default=key)
        click.echo(f"You Entered: {newkey}\n")
        if click.confirm('Do you want to save?',abort=True,default=True):
            data.pop(key)
            temp = newkey
            while temp in data:
                temp+="d"
            if temp != newkey:
                data[temp]=data.pop(newkey)
            data[newkey]={'name':name,'command':command}

    TL_CONFIG.saveData(data)

    click.echo(td(title="",text=f"<g> {ops.capitalize()} to '{name}' was saved.</g>"))




if __name__ == "__main__":
    toollist()



import click
import math
import re
from cli.utils import TableDisplay
# init()

@click.command()
@click.argument("inputs",nargs=-1)
@click.option('--precision','-p','precision',default='9g',help="Precision,9g/9e/9f",show_default=True)
def cli(inputs,precision):
    """
    A big calculator.
    """
    # precision handing:
    precision = precision.strip('. ')

    formula = "".join(inputs)
    # replace log and ln
    formula = re.sub( "log\s*(?P<num>\d+)" , "math.log10(\g<num>)" , formula)
    formula = re.sub( "log\s*\((?P<num>.*)\)" , "math.log10(\g<num>)" , formula)
    formula = re.sub( "ln\s*(?P<num>\d+)" , "math.log(\g<num>)" , formula)
    formula = re.sub( "ln\s*\((?P<num>.*)\)" , "math.log(\g<num>)" , formula)

    # replace constant e and pi
    formula = re.sub( "eu" , "math.e" , formula)
    formula = re.sub( "pi" , "math.pi" , formula)

    # replace ^ by **
    formula = re.sub( "\^" , "**" , formula)
    formula = re.sub( "sqrt" , "math.sqrt" , formula)
    formula = re.sub( "sin" , "math.sin" , formula)
    formula = re.sub( "cos" , "math.cos" , formula)
    formula = re.sub( "tan" , "math.tan" , formula)
    formula = re.sub( "asin" , "math.asin" , formula)
    formula = re.sub( "acos" , "math.acos" , formula)
    formula = re.sub( "atan" , "math.atan" , formula)
    formula = re.sub( "abs" , "math.fabs" , formula)

    # replace factorial inside parenthesis
    formula = re.sub("\((?P<num>.*)\)!","math.factorial(\g<num>)",formula,)
    # replace factorial of numbers
    formula = re.sub("(?P<num>\s*\d+\s*)!","math.factorial(\g<num>)",formula)
    try:
        result = eval(formula)
        result = [(f'  [结果] = {{:.{precision}}}').format(result)]

    except Exception as e:
        result = [f"Formula : {formula}" , '<alert>Error {} </alert>'.format(e)]
    line1 = f'  [计算] <f>{"".join(inputs)}</f>'
    tf = TableDisplay({"[]":"fGsB",("<f>","</f>"):"fMsB","{}":"bB",("<alert>","</alert>"):"bR"})
    data = tf(title='{>>>计算器<<<}',text=[[line1],result])
    click.echo('')
    click.echo(data)
    click.echo('')
